@@ -1,4 +1,4 @@
-# Copyright 2018-2019 Descartes Labs.
+# Copyright 2018-2020 Descartes Labs.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -207,9 +207,9 @@ class SceneCollection(Collection):
             or are invalid.
             If not all required parameters are specified in the :class:`~descarteslabs.scenes.geocontext.GeoContext`.
             If the SceneCollection is empty.
-        `NotFoundError`
+        NotFoundError
             If a Scene's ID cannot be found in the Descartes Labs catalog
-        `BadRequestError`
+        BadRequestError
             If the Descartes Labs platform is given unrecognized parameters
         """
         if len(self) == 0:
@@ -235,7 +235,7 @@ class SceneCollection(Collection):
             )  # the bands axis for each component ndarray call in the stack
 
         if flatten is not None:
-            if isinstance(flatten, six.string_types) or not hasattr(flatten, "__len__"):
+            if isinstance(flatten, str) or not hasattr(flatten, "__len__"):
                 flatten = [flatten]
             scenes = [
                 sc if len(sc) > 1 else sc[0] for group, sc in self.groupby(*flatten)
@@ -248,18 +248,10 @@ class SceneCollection(Collection):
         if raster_info:
             raster_infos = [None] * len(scenes)
 
-        alpha_band_name = "alpha"
-        if isinstance(mask_alpha, six.string_types):
-            alpha_band_name = mask_alpha
-        elif mask_alpha is None:
-            mask_alpha = self._collection_has_alpha(alpha_band_name)
-
         bands = Scene._bands_to_list(bands)
-        pop_alpha = False
-        if mask_alpha and alpha_band_name not in bands:
-            pop_alpha = True
-            bands.append(alpha_band_name)
-            scaling = _scaling.append_alpha_scaling(scaling)
+        (bands, scaling, mask_alpha, pop_alpha) = self._mask_alpha_if_applicable(
+            bands, mask_alpha=mask_alpha, scaling=scaling
+        )
 
         scales, data_type = _scaling.multiproduct_scaling_parameters(
             self._product_band_properties(), bands, scaling, data_type
@@ -424,9 +416,9 @@ class SceneCollection(Collection):
             or are invalid.
             If not all required parameters are specified in the :class:`~descarteslabs.scenes.geocontext.GeoContext`.
             If the SceneCollection is empty.
-        `NotFoundError`
+        NotFoundError
             If a Scene's ID cannot be found in the Descartes Labs catalog
-        `BadRequestError`
+        BadRequestError
             If the Descartes Labs platform is given unrecognized parameters
         """
         if len(self) == 0:
@@ -440,30 +432,13 @@ class SceneCollection(Collection):
             )
 
         bands = Scene._bands_to_list(bands)
-        alpha_band_name = "alpha"
-        if isinstance(mask_alpha, six.string_types):
-            alpha_band_name = mask_alpha
-        elif mask_alpha is None:
-            mask_alpha = self._collection_has_alpha(alpha_band_name)
-
-        if mask_alpha:
-            try:
-                alpha_i = bands.index(alpha_band_name)
-            except ValueError:
-                bands.append(alpha_band_name)
-                drop_alpha = True
-                scaling = _scaling.append_alpha_scaling(scaling)
-            else:
-                if alpha_i != len(bands) - 1:
-                    raise ValueError(
-                        "Alpha must be the last band in order to reduce rasterization errors"
-                    )
-                drop_alpha = False
+        (bands, scaling, mask_alpha, drop_alpha) = self._mask_alpha_if_applicable(
+            bands, mask_alpha=mask_alpha, scaling=scaling
+        )
 
         scales, data_type = _scaling.multiproduct_scaling_parameters(
             self._product_band_properties(), bands, scaling, data_type
         )
-
         raster_params = ctx.raster_params
         full_raster_args = dict(
             inputs=[scene.properties["id"] for scene in self],
@@ -643,9 +618,9 @@ class SceneCollection(Collection):
             If ``format`` is invalid, or a path has an invalid extension.
         TypeError
             If ``dest`` is not a string or a sequence type.
-        `NotFoundError`
+        NotFoundError
             If a Scene's ID cannot be found in the Descartes Labs catalog
-        `BadRequestError`
+        BadRequestError
             If the Descartes Labs platform is given unrecognized parameters
         """
         if len(self) == 0:
@@ -709,8 +684,8 @@ class SceneCollection(Collection):
         download_args = dict(
             resampler=resampler,
             processing_level=processing_level,
-            scales=scales,
-            dtype=data_type,
+            scaling=scales,
+            data_type=data_type,
             raster_client=self._raster_client,
         )
         try:
@@ -723,13 +698,22 @@ class SceneCollection(Collection):
                 scene.download(bands, ctx, dest=path, **download_args)
         else:
             with futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
-                futures = [
+                futures = {
                     executor.submit(
                         scene.download, bands, ctx, dest=path, **download_args
-                    )
+                    ): path
                     for scene, path in zip(self, dest)
-                ]
-                concurrent.futures.wait(futures)
+                }
+                exceptions = []
+                for future in concurrent.futures.as_completed(futures):
+                    try:
+                        future.result()
+                    except Exception as ex:
+                        exceptions.append((futures[future], ex))
+                if exceptions:
+                    raise RuntimeError(
+                        "One or more downloads failed: {}".format(exceptions)
+                    )
         return dest
 
     def download_mosaic(
@@ -742,6 +726,7 @@ class SceneCollection(Collection):
         processing_level=None,
         scaling=None,
         data_type=None,
+        mask_alpha=None,
     ):
         """
         Download all scenes as a single image file.
@@ -792,6 +777,12 @@ class SceneCollection(Collection):
         data_type : None, str
             Output data type. Please see :meth:`scaling_parameters` for a full
             description of this parameter.
+        mask_alpha : bool or str or None, default None
+            Whether to mask pixels in all bands where the alpha band of all scenes is 0.
+            Provide a string to use an alternate band name for masking.
+            If the alpha band is available for all scenes in the collection and
+            ``mask_alpha`` is None, ``mask_alpha`` is set to True. If not,
+            mask_alpha is set to False.
 
         Returns
         -------
@@ -820,15 +811,18 @@ class SceneCollection(Collection):
             If not all required parameters are specified in the :class:`~descarteslabs.scenes.geocontext.GeoContext`.
             If the SceneCollection is empty.
             If ``format`` is invalid, or the path has an invalid extension.
-        `NotFoundError`
+        NotFoundError
             If a Scene's ID cannot be found in the Descartes Labs catalog
-        `BadRequestError`
+        BadRequestError
             If the Descartes Labs platform is given unrecognized parameters
         """
         if len(self) == 0:
             raise ValueError("This SceneCollection is empty")
 
         bands = Scene._bands_to_list(bands)
+        (bands, scaling, mask_alpha, drop_alpha) = self._mask_alpha_if_applicable(
+            bands, mask_alpha=mask_alpha, scaling=scaling
+        )
         scales, data_type = _scaling.multiproduct_scaling_parameters(
             self._product_band_properties(), bands, scaling, data_type
         )
@@ -936,6 +930,28 @@ class SceneCollection(Collection):
     def _collection_has_alpha(self, alpha_band_name):
         return all(scene.has_alpha(alpha_band_name) for scene in self)
 
+    def _mask_alpha_if_applicable(self, bands, mask_alpha=None, scaling=None):
+        alpha_band_name = "alpha"
+        if isinstance(mask_alpha, str):
+            alpha_band_name = mask_alpha
+        elif mask_alpha is None:
+            mask_alpha = self._collection_has_alpha(alpha_band_name)
+
+        drop_alpha = False
+        if mask_alpha:
+            try:
+                alpha_i = bands.index(alpha_band_name)
+            except ValueError:
+                bands.append(alpha_band_name)
+                drop_alpha = True
+                scaling = _scaling.append_alpha_scaling(scaling)
+            else:
+                if alpha_i != len(bands) - 1:
+                    raise ValueError(
+                        "Alpha must be the last band in order to reduce rasterization errors"
+                    )
+        return (bands, scaling, mask_alpha, drop_alpha)
+
     def _product_band_properties(self):
         result = {}
         for scene in self:
